@@ -1,6 +1,6 @@
 #############################################################################
 #  
-#	$Id: core.py,v 2.104.2.19 2008/08/19 14:30:38 irmen Exp $
+#	$Id: core.py,v 2.104.2.30 2009/12/06 23:55:26 irmen Exp $
 #	Pyro Core Library
 #
 #	This is part of "Pyro" - Python Remote Objects
@@ -39,7 +39,7 @@ def _checkInit(pyrotype="client"):
 #
 #############################################################################
 
-class ObjBase:
+class ObjBase(object):
 	def __init__(self):
 		self.objectGUID=Pyro.util.getGUID()
 		self.delegate=None
@@ -253,33 +253,19 @@ class CallbackObjBase(ObjBase):
 #
 #############################################################################
 
-class PyroURI:
+class PyroURI(object):
 	def __init__(self,host,objectID=0,port=0,prtcol='PYRO'):
 		# if the 'host' arg is a PyroURI, copy contents
 		if isinstance(host, PyroURI):
-			self.reinitFromString(str(host))
+			self.init(host.address, host.objectID, host.port, host.protocol)
 		else:
 			# If the 'host' arg contains '://', assume it's an URI string.
 			if host.find('://')>0:
 				self.reinitFromString(host)
 			else:
-				if '/' in host:
-					raise URIError('malformed hostname')
-				if Pyro.config.PYRO_DNS_URI:
-					self.address = host
-				else:
-					self.address=Pyro.protocol.getIPAddress(host)
-					if not self.address:
-						raise URIError('unknown host')
-				if port:
-					if type(port)==type(1):
-						self.port=port
-					else:
-						raise TypeError("port must be integer")
-				else:
-					self.port=Pyro.config.PYRO_PORT
-				self.protocol=prtcol
-				self.objectID=objectID
+				if not objectID:
+					raise URIError('invalid URI format')
+				self.init(host, objectID, port, prtcol)
 	def __str__(self):
 		return self.protocol+'://'+self.address+':'+str(self.port)+'/'+self.objectID
 	def __repr__(self):
@@ -291,14 +277,24 @@ class PyroURI:
 		return cmp(str(self), str(o))
 	def clone(self):
 		return PyroURI(self)
-	def init(self,address,objectID,port=0,prtcol='PYRO'):
-		self.address=address
-		self.objectID=objectID
+	def init(self,host,objectID,port=0,prtcol='PYRO'):
+		if '/' in host:
+			raise URIError('malformed hostname')
+		if Pyro.config.PYRO_DNS_URI:
+			self.address = host
+		else:
+			self.address=Pyro.protocol.getIPAddress(host)
+			if not self.address:
+				raise URIError('unknown host')
 		if port:
-			self.port=port
+			if type(port)==type(1):
+				self.port=port
+			else:
+				raise TypeError("port must be integer")
 		else:
 			self.port=Pyro.config.PYRO_PORT
 		self.protocol=prtcol
+		self.objectID=objectID
 	def reinitFromString(self,arg):
 		if arg.startswith('PYROLOC') or arg.startswith('PYRONAME'):
 			uri=processStringURI(arg)
@@ -306,17 +302,13 @@ class PyroURI:
 			return
 		x=re.match(r'(?P<protocol>[^\s:/]+)://(?P<hostname>[^\s:]+):?(?P<port>\d+)?/(?P<id>\S*)',arg)
 		if x:
-			self.protocol=x.group('protocol')
-			self.address=x.group('hostname')
-			self.port=x.group('port')
-			if self.port:
-				self.port=int(self.port)
-			else:
-				self.port=Pyro.config.PYRO_PORT
-			self.objectID=x.group('id')
+			port=None
+			if x.group('port'):
+				port=int(x.group('port'))
+			self.init(x.group('hostname'), x.group('id'), port, x.group('protocol'))
 			return
-		Log.error('PyroURI','illegal URI format passed: '+arg)
-		raise URIError('illegal URI format')
+		Log.error('PyroURI','invalid URI format passed: '+arg)
+		raise URIError('invalid URI format')
 	def getProxy(self):
 		return DynamicProxy(self)
 	def getAttrProxy(self):
@@ -380,7 +372,7 @@ def getProxyForURI(URI):
 def getAttrProxyForURI(URI):
 	return DynamicProxyWithAttrs(URI)
 
-class _RemoteMethod:
+class _RemoteMethod(object):
 	# method call abstraction, adapted from Python's xmlrpclib
 	# it would be rather easy to add nested method calls, but
 	# that is not compatible with the way that Pyro's method
@@ -391,7 +383,7 @@ class _RemoteMethod:
 	def __call__(self, *args, **kwargs):
 		return self.__send(self.__name, args, kwargs)
 
-class DynamicProxy:
+class DynamicProxy(object):
 	def __init__(self, URI):
 		_checkInit() # init required
 		if type(URI) in (StringType,UnicodeType):
@@ -422,6 +414,10 @@ class DynamicProxy:
 	def _release(self):
 		if self.adapter:
 			self.adapter.release()
+	def _local(self):
+		return self.URI._local()
+	def _islocal(self):
+		return self.URI._islocal()
 	def __copy__(self):			# create copy of current proxy object
 		proxyCopy = DynamicProxy(self.URI)
 		proxyCopy.adapter.setIdentification(self.adapter.getIdentification(), munge=False)   # copy identification info
@@ -432,7 +428,7 @@ class DynamicProxy:
 	def __deepcopy__(self, arg):
 		raise PyroError("cannot deepcopy a proxy")
 	def __getattr__(self, name):
-		if name=="__getinitargs__":			# allows it to be safely pickled
+		if name in ("__getnewargs__","__getinitargs__"):		# allows it to be safely pickled
 			raise AttributeError()
 		return _RemoteMethod(self._invokePYRO, name)
 	def __repr__(self):
@@ -458,24 +454,24 @@ class DynamicProxy:
 
 	def _invokePYRO(self, name, vargs, kargs):
 		if not self.adapter.connected():
+			# rebind here, don't do it from inside the remoteInvocation because deadlock will occur
 			self.adapter.bindToURI(self.URI)
 		return self.adapter.remoteInvocation(name, Pyro.constants.RIF_VarargsAndKeywords, vargs, kargs)
 
 	# Pickling support, otherwise pickle uses __getattr__:
 	def __getstate__(self):
 		# for pickling, return a non-connected copy of ourselves:
-		copy = self.__copy__()
-		copy._release()
-		return copy.__dict__
+		cpy = self.__copy__()
+		cpy._release()
+		return cpy.__dict__
 	def __setstate__(self, args):
-		# this appears to be necessary otherwise pickle won't work
-		self.__dict__=args
+		# for pickling, to restore the pickled state
+		self.__dict__.update(args)
 
 	
 class DynamicProxyWithAttrs(DynamicProxy):
+	_local_attrs = ("_local_attrs","URI", "objectID", "adapter", "_attr_cache")
 	def __init__(self, URI):
-		# first set the list of 'local' attrs for __setattr__
-		self.__dict__["_local_attrs"] = ("_local_attrs","URI", "objectID", "adapter", "_name", "_attr_cache")
 		self._attr_cache = {}
 		DynamicProxy.__init__(self, URI)
 	def _r_ga(self, attr, value=0):
@@ -492,7 +488,7 @@ class DynamicProxyWithAttrs(DynamicProxy):
 	def __copy__(self):		# create copy of current proxy object
 		return DynamicProxyWithAttrs(self.URI)
 	def __setattr__(self, attr, value):
-		if attr in self.__dict__["_local_attrs"]:
+		if attr in self._local_attrs:
 			self.__dict__[attr]=value
 		else:
 			result = self.findattr(attr)
@@ -502,7 +498,7 @@ class DynamicProxyWithAttrs(DynamicProxy):
 				raise AttributeError('not an attribute')
 	def __getattr__(self, attr):
 		# allows it to be safely pickled
-		if attr not in ("__getinitargs__", "__hash__","__eq__","__ne__") and attr not in self.__dict__["_local_attrs"]:
+		if attr not in ("__getnewargs__","__getinitargs__", "__hash__","__eq__","__ne__") and attr not in self._local_attrs:
 			result=self.findattr(attr)
 			if result==1: # method
 				return _RemoteMethod(self._invokePYRO, attr)
@@ -670,22 +666,22 @@ class Daemon(Pyro.protocol.TCPServer, ObjBase):
 		return DynamicProxyWithAttrs( PyroURI(self.hostname,
 				obj.GUID(), prtcol=self.protocol, port=self.port) )
 
-	def connectPersistent(self, object, name=None):
+	def connectPersistent(self, obj, name=None):
 		# when a persistent entry is found in the NS, that URI is
 		# used instead of the supplied one, if the address matches.
 		if name and self.NameServer:
 			self.nscallMutex.acquire()
 			try:
 				try:
-					newURI = PyroURI(self.hostname, object.GUID(), prtcol=self.protocol, port=self.port)
+					newURI = PyroURI(self.hostname, obj.GUID(), prtcol=self.protocol, port=self.port)
 					URI=self.NameServer.resolve(name)
 					if (URI.protocol,URI.address,URI.port)==(newURI.protocol,newURI.address,newURI.port):
 						# reuse the previous object ID
-						object.setGUID(URI.objectID)
+						obj.setGUID(URI.objectID)
 						# enter the (object,name) in the known impl. dictionary
-						self.implementations[object.GUID()]=(object,name)
-						self.persistentConnectedObjs.append(object.GUID())
-						object.setPyroDaemon(self)
+						self.implementations[obj.GUID()]=(obj,name)
+						self.persistentConnectedObjs.append(obj.GUID())
+						obj.setPyroDaemon(self)
 						return URI
 					else:
 						# name exists, but address etc. is wrong. Remove it.
@@ -697,11 +693,11 @@ class Daemon(Pyro.protocol.TCPServer, ObjBase):
 			finally:
 				self.nscallMutex.release()
 		# Register normally.		
-		self.persistentConnectedObjs.append(object.GUID())
-		return self.connect(object, name)
+		self.persistentConnectedObjs.append(obj.GUID())
+		return self.connect(obj, name)
 
-	def connect(self, object, name=None):
-		URI = PyroURI(self.hostname, object.GUID(), prtcol=self.protocol, port=self.port)
+	def connect(self, obj, name=None):
+		URI = PyroURI(self.hostname, obj.GUID(), prtcol=self.protocol, port=self.port)
 		# if not transient, register the object with the NS
 		if name:
 			self.nscallMutex.acquire()
@@ -713,26 +709,33 @@ class Daemon(Pyro.protocol.TCPServer, ObjBase):
 			finally:
 				self.nscallMutex.release()
 		# enter the (object,name) in the known implementations dictionary
-		self.implementations[object.GUID()]=(object,name)
-		object.setPyroDaemon(self)
+		self.implementations[obj.GUID()]=(obj,name)
+		obj.setPyroDaemon(self)
 		return URI
 
-	def disconnect(self,object):
+	def disconnect(self,obj):		# obj can be either the object that was registered, or its uid
 		try:
-			if self.NameServer and self.implementations[object.GUID()][1]:
+			if isinstance(obj,Pyro.core.ObjBase):
+				obj_uid=obj.GUID()
+			else:
+				obj_uid=str(obj)
+			if obj_uid==Pyro.constants.INTERNAL_DAEMON_GUID:
+				return   # never allow to remove ourselves from the registry
+			if self.NameServer and self.implementations[obj_uid][1]:
 				self.nscallMutex.acquire()
 				try:
 					# only unregister with NS if it had a name (was not transient)
-					self.NameServer.unregister(self.implementations[object.GUID()][1])
+					self.NameServer.unregister(self.implementations[obj_uid][1])
 				finally:
 					self.nscallMutex.release()
-			del self.implementations[object.GUID()]
-			if object.GUID() in self.persistentConnectedObjs:
-				self.persistentConnectedObjs.remove(object.GUID())
+			del self.implementations[obj_uid]
+			if obj_uid in self.persistentConnectedObjs:
+				self.persistentConnectedObjs.remove(obj_uid)
 			# XXX Clean up connections/threads to this object?
 			#     Can't be done because thread/socket is not associated with single object 
 		finally:
-			object.setPyroDaemon(None)
+			if isinstance(obj,Pyro.core.ObjBase):
+				obj.setPyroDaemon(None)
 
 	def getRegistered(self):
 		r={}
@@ -878,3 +881,6 @@ def initServer(banner=0, storageCheck=1):
 		print 'Pyro Server Initialized. Using Pyro V'+Pyro.constants.VERSION
 	_init_server_done=1
 
+
+if __name__=="__main__":
+	print "Pyro version:",Pyro.constants.VERSION
